/* 
Unlimited Simulator is copyrighted (c) Bruno Barberi Gnecco, 1999, and
distributed under the GNU General Public License (GPL). The conditions can be
found in the file GPL, resident at the main directory of the distribution.
*/
#ifndef MATHLIB_H
#define MATHLIB_H

#include <math.h>

#define PIOVER2		1.57079632679489661923132169164		/* PI/2 */
#define TWOOVERPI	0.63661977236758134307553505349
#define PI3OVER2	4.71238898038468985769396507492		/* 3*PI/2 */
#ifndef PI
 #define PI		3.14159265358979323846264338328
#endif
#ifndef PI2
 #define PI2		6.28318530717958647692528676656		/* 2*PI */
#endif

/* it seems that rint is not completely standard. anyway, rounding could be
faster if written in assembler, but this should be done for each platform... */
#define	round( d )	((int)rint( d ))

struct vector_ul {
	double 		x, y, z;
};
typedef struct vector_ul VECTOR_UL;

struct point_ul {
	VECTOR_UL	*vect;
	VECTOR_UL	*norm;		/* normal, if point belongs to a 
					   surface. Declaring it as a pointer
					   saves space! */
#ifdef CLIENT
	float		u, v;		/* texture mapping coordinates */
#endif

};
typedef struct point_ul POINT_UL;

struct quaternion_ul {
	double s;
	VECTOR_UL v;
};
typedef struct quaternion_ul QUATERNION_UL;

/* a 3x3 matrix to change basis */
typedef double BASISCHANGE_UL[3][3];

typedef double MATRIX_UL[4][4];

struct plane_ul {
	VECTOR_UL 	normal;
	double 		dist; 	/* shortest distance from origin, world coord */
	double 		a, b, c, d;
	VECTOR_UL	p;	/* a point in the plane */	
};
typedef struct plane_ul PLANE_UL;

struct sphere_ul {
	double radius;
	VECTOR_UL center;
};
typedef struct sphere_ul SPHERE_UL;

/* vector.c */
extern void initV ( VECTOR_UL *vect, double x, double y, double z );
extern void zeroV ( VECTOR_UL *vect );
extern double normV ( VECTOR_UL *vect );
extern void copyV ( VECTOR_UL *target, const VECTOR_UL *src );
extern void copyPtoV ( VECTOR_UL *target, const POINT_UL *src );
extern void copyVtoP ( POINT_UL *target, const VECTOR_UL *src );
extern void addV ( VECTOR_UL *vect1, VECTOR_UL *vect2, VECTOR_UL *result );
extern void subV ( VECTOR_UL *vect1, VECTOR_UL *vect2, VECTOR_UL *result );
extern void scalarMV ( VECTOR_UL *vect, const double scalar, VECTOR_UL *result );
extern void normalizeV ( VECTOR_UL *vect );
extern double dotprodV ( VECTOR_UL *vect1, VECTOR_UL *vect2 );
extern void crossprodV ( VECTOR_UL *vect1, VECTOR_UL *vect2, VECTOR_UL *result );
extern double dist ( VECTOR_UL *v1, VECTOR_UL *v2 );
extern double sqdist ( VECTOR_UL *v1, VECTOR_UL *v2 );
extern void multM3 ( BASISCHANGE_UL a, BASISCHANGE_UL b, BASISCHANGE_UL c );
extern void changebasis ( VECTOR_UL *v, BASISCHANGE_UL b, VECTOR_UL *r );
extern void invertbasis ( BASISCHANGE_UL b, BASISCHANGE_UL r );
extern void basisVA ( VECTOR_UL *v, double angle, BASISCHANGE_UL m );		 

/* quat.c */
extern void initQuatD ( QUATERNION_UL *q, double s, double x, double y, double z );
extern void initQuatVS ( QUATERNION_UL *q, double s, VECTOR_UL *v );
extern void initQuatVA ( QUATERNION_UL *q, double angle, VECTOR_UL *v );
extern void zeroQ ( QUATERNION_UL *q );
extern void copyQ ( QUATERNION_UL *dest, QUATERNION_UL *src );
extern double normQ ( QUATERNION_UL *q );
extern void normalizeQ ( QUATERNION_UL *q, QUATERNION_UL *r );
extern void conjugateQ ( QUATERNION_UL *q, QUATERNION_UL *r );
extern void addQ ( QUATERNION_UL *q1, QUATERNION_UL *q2, QUATERNION_UL *r );
extern void subQ ( QUATERNION_UL *q1, QUATERNION_UL *q2, QUATERNION_UL *r );
extern void scalarMQ( QUATERNION_UL *q, const double m, QUATERNION_UL *r);
extern void multQ ( QUATERNION_UL *q1, QUATERNION_UL *q2, QUATERNION_UL *r );
extern void inverseQ ( QUATERNION_UL *q1, QUATERNION_UL *r );
extern void qvq ( QUATERNION_UL *q1, VECTOR_UL *v, QUATERNION_UL *q2, 
		VECTOR_UL *r );

/* linsys.c */
#define linsys( a, b, c, d )	gausselim( a, b, c, d )
extern int gausselim ( int n, double A[n][n], double B[n], double x[n] );

/* matrix.c */
extern void identity ( MATRIX_UL a );
extern void addM ( MATRIX_UL a, MATRIX_UL b, MATRIX_UL r );
extern void subM ( MATRIX_UL a, MATRIX_UL b, MATRIX_UL r );
extern void scalarmM ( MATRIX_UL a, const double m, MATRIX_UL r );
extern void multM ( MATRIX_UL a, MATRIX_UL b, MATRIX_UL r );
extern void multVM ( MATRIX_UL m, VECTOR_UL *v, VECTOR_UL *result );

/* analytic.c */
extern void initPlaneP ( PLANE_UL *p, POINT_UL *p1, POINT_UL *p2, POINT_UL *p3 );
extern void initPlaneVD ( PLANE_UL *p, VECTOR_UL *v, double d );
extern int interPlaneLine ( PLANE_UL *p, VECTOR_UL *v1, VECTOR_UL *v2,
		 VECTOR_UL *r );

/* trig.c */
extern double TrigTable[];
extern void initTrigTable ( void );
extern double sinus ( double rad );
extern double sinusd ( double deg );
extern double cosinus ( double rad );
extern double cosinusd (double deg );

#endif
