/* devdsp.c */
/* This program is placed in the Public Domain by Steve Kirkendall */

/* This file contains Linux-specific version of the devopen() function, which
 * opens the sound port with a given configuration, and returns the file
 * descriptor.
 */

#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <linux/soundcard.h>
#include "unixsnd.h"

#ifndef AUDIO
# define AUDIO		"/dev/dsp"
#endif
#ifndef AFMT_S16_NE
# define AFMT_S16_NE AFMT_S16_LE
#endif

#define check(ret, name)	if ((ret) < 0) {\
					fprintf(stderr, "unixsnd: could not configure the %s\n", name);\
					return -1;\
				} else

/* Test a device for 16 bit capability */
int devbits(char *name)
{
	int	fd;
	int	ret;

	/* Open the default port */
	fd = open(name ? name : AUDIO, O_WRONLY);
	if (fd < 0)
		return -1;

	/* test an attribute */
	ret = AFMT_S16_LE;
	ret = ioctl(fd, SNDCTL_DSP_SETFMT, &ret) < 0 ? 8 : 16;

	/* close the port */
	close(fd);
	return ret;
}

/* Test a device for stereo capability */
int devstereo(char *name)
{
	int	fd;
	int	ret;

	/* Open the default port */
	fd = open(name ? name : AUDIO, O_WRONLY);
	if (fd < 0)
		return -1;

	/* test for stereo capability */
	ret = 1;
	ret = ioctl(fd, SNDCTL_DSP_STEREO, &ret) < 0 ? 0 : 1;

	/* close the port */
	close(fd);
	return ret;
}


/* Test a device for sample frequency.  Actually, we just return 11025 here
 * for all cases.
 */
int devfrequency(char *name, int stereo, int bits)
{
	return 11025;
}


/* Open the sound port, and configure it for mono/stereo, and 8/16 bit signed
 * samples.
 */
int devopen(char *name, int stereo, int bits, int frequency)
{
	int	fd;
	int	encoding;
	int	frags;

	/* if no device specified, then use the default */
	if (!name)
		name = AUDIO;

	/* Try to open the audio device */
 	fd = open(name, O_WRONLY);
 	if (fd < 0)
 	{
 		perror(name);
 		return -1;
 	}

	/* set the output buffer size */
	frags = 0x00030009;
	if (stereo)
		frags++;
	if (bits == 16)
		frags++;
	check(ioctl(fd, SNDCTL_DSP_SETFRAGMENT, &frags), "kernel's sound buffers");
	/* set 8 or 16 bits, signed */
	encoding = (bits == 8 ? AFMT_S8 : AFMT_S16_NE);
	check(ioctl(fd, SNDCTL_DSP_SETFMT, &encoding), "number of bits (-b)");

	/* set stereo or mono */
	check(ioctl(fd, SNDCTL_DSP_STEREO, &stereo), "number of channels (-s/-m)");

	/* set the rate */
	check(ioctl(fd, SNDCTL_DSP_SPEED, &frequency), "sample rate (-r)");

	return fd;
}


/* force all sound data to be output, even if the kernel buffer isn't full */
void devflush(int fd)
{
	(void)ioctl(fd, SNDCTL_DSP_POST, 0);
}
